###############################################################################
# Tests for chemkin.thermodynamics.thermo module
###############################################################################

from chemkin.thermodynamics.thermo import ThermoDAO

def test_get_coeffs():
	dao = ThermoDAO('NASA_coef.sqlite')
	high_coefs = dao.get_coeffs('H', 'high')
	low_coefs = dao.get_coeffs('H', 'low')
	assert len(high_coefs) == 7
	assert len(low_coefs) == 7


def test_get_species():
	dao = ThermoDAO('NASA_coef.sqlite')
	species_high = dao.get_species(1001, 'high')
	species_low = dao.get_species(999, 'low')
	assert len(species_high) > 0
	assert len(species_low) > 0



import numpy as np
from chemkin.thermodynamics.thermo import ThermoDAO
from chemkin.chemkin_errors import ChemKinError

class RxnCoefficientBase():
    """Base class of reaction rate coefficients

	ATTRIBUTES:
	========
	k: float
		Reaction rate coefficient
		Initialized to None in base class and should be calculated and
		returned in its subclass.

	METHODS:
	========
	get_coef(): Calculates and returns the reaction rate coefficient
		This should be implemented in its subclass.
	"""

    def __init__ (self):
        self.k = None

    def __repr__ (self):
        return 'RxnCoefficientBase()'

    def __eq__ (self, other):
        """ Returns the true if 2 coefficients have the same value"""
        if self.get_coef() == other.get_coef():
            return True
        else:
            return False

    def get_coef (self):
        """Calculates and returns the reaction rate coefficient

        Method is not implemented in this base class, but should be
        implemented in its subclasses.
        """
        raise NotImplementedError('Subclass must implement this method')


class ConstantCoefficient(RxnCoefficientBase):
    """Class of constant reaction rate coefficients
	Subclass of RxnCoefficientBase

	ATTRIBUTES:
	========
	k: float, required
		Reaction rate coefficient
		Initialized with constructor argument k

	METHODS:
	========
	get_coef(): Returns the constant rate coefficient
	"""

    def __init__ (self, k):
        """
        NOTES
        =====
        PRE:
            - k have numeric type
            - one input
        """
        self.k = k

    def __repr__ (self):
        return 'ConstantCoefficient(k = {})'.format(self.k)

    def get_coef (self):
        """Returns the constant rate coefficient

        RETURNS:
        ========
        self.k: float
            Constant reaction rate coefficient
        NOTES
        =====
        POST:
             - self.k is not changed by this function
             - raises a ValueError exception if k <= 0
             - returns a float of self.k

        EXAMPLES:
        =========
        >>> ConstantCoefficient(10.0).get_coef()
        10.0
        """
        if self.k < 0:
            raise ValueError(
                  "Negative reaction rate coefficients are prohibited.")
        return self.k


class ArrheniusCoefficient(RxnCoefficientBase):
    """Class of Arrhenius reaction rate coefficient
	Subclass of RxnCoefficientBase

	ATTRIBUTES:
	========
	k: float
		Reaction rate coefficient
		Calculated by self.get_coef()
	A: float, required
		Arrhenius prefactor
		Initialized with constructor argument A
	E: float, required
		Activation energy
		Initialized with constructor argument E
	T: float, required
		Temperature (in Kelvin)
		Initialized with constructor argument T
	R: float, optional, default value = 8.314
		Ideal gas constant
		Initialized with constructor argument R
		Default value of R should not be changed except for unit conversion

	METHODS:
	========
	get_coef(): Calculates and returns the Arrhenius rate coefficient
	"""

    def __init__ (self, A, E, T, R=8.314):

        """
        NOTES
        =====
        PRE:
            - A, E, T and R have numeric type
            - four or fewer inputs
        """
        super().__init__()
        self.A = A
        self.E = E
        self.T = T
        self.R = R

    def __repr__ (self):
        return 'ArrheniusCoefficient(A={}, E={}, T={}, R={})'.format(self.A, self.E,
                                                              self.T, self.R)

    def get_coef (self):
        """ Method to calculate the Arrhenius reaction rate coefficient

        RETURNS:
        ========
        self.k: float
            Arrhenius reaction rate coefficient

        NOTES
        =====
        POST:
             - self.A, self.E, self.T and self.R are not changed by this
             function
             - raises a ValueError exception if A <= 0 or T <= 0
             - returns a float of self.k

        EXAMPLES:
        =========
        >>> ArrheniusCoefficient(2.0, 3.0, 100.0).get_coef()
        1.9927962618542914
        """
        if self.A < 0.0:
            raise ValueError(
                  "A = {0:18.16e}:  Negative Arrhenius prefactor is "
                  "prohibited!".format(
                        self.A))

        if self.T < 0.0:
            raise ValueError(
                  "T = {0:18.16e}:  Negative temperatures are "
                  "prohibited!".format(
                        self.T))

        if self.R < 0.0:
            raise ValueError(
                  "R = {0:18.16e}:  Negative ideal gas constant is "
                  "prohibited!".format(
                        self.R))

        else:
            try:
                self.k = self.A * np.exp(-self.E / (self.R * self.T))
            except Warning:
                raise OverflowError("The result is too large/small.")
            return self.k


class ModifiedArrheniusCoefficient(ArrheniusCoefficient):
    """Class of Modified Arrhenius reaction rate coefficient
		Subclass of ArrheniusCoefficient

		ATTRIBUTES:
		========
		k: float
			Reaction rate coefficient
			Calculated by self.get_coef()
		A: float, required
			Arrhenius prefactor
			Initialized with constructor argument A
		b: float, required
			Modified Arrhenius parameter
			Initialized with constructor argument b
		E: float, required
			Activation energy
			Initialized with constructor argument E
		T: float, required
			Temperature (in Kelvin)
			Initialized with constructor argument T
		R: float, optional, default value = 8.314
			Ideal gas constant
			Initialized with constructor argument R
			Default value of R should not be changed except for unit conversion

		METHODS:
		========
		get_coef(): Calculates and returns the Modified Arrhenius rate
		coefficient
	"""

    def __init__ (self, A, b, E, T, R=8.314):

        """
		NOTES
		=====
		PRE:
			- A, b, E, T and R have numeric type
			- five or fewer inputs
		"""
        super().__init__(A, E, T, R)
        self.b = b

    def __repr__ (self):
        return 'ModifiedArrheniusCoefficient(A={}, b={}, E={}, T={}, R={})'.format(self.A,
                                                                       self.b,
                                                                       self.E,
                                                                       self.T,
                                                                       self.R)

    def get_coef (self):
        """ Method to calculate the Modified Arrhenius reaction rate coefficient

        RETURNS:
        ========
        self.k: float
        Modified Arrhenius reaction rate coefficient

        NOTES
        =====
        POST:
             - self.A, self.E, self.T and self.R are not changed by this
             function
             - raises a ValueError exception if A <= 0, T <= 0 or b is not a
             real number
             - returns a float of self.k

        EXAMPLES:
        =========
        >>> ModifiedArrheniusCoefficient(2.0, -0.5, 3.0, 100.0).get_coef()
        0.19927962618542916
        """
        if self.A < 0.0:
            raise ValueError(
                  "A = {0:18.16e}:  Negative Arrhenius prefactor is "
                  "prohibited!".format(
                        self.A))

        if self.T < 0.0:
            raise ValueError(
                  "T = {0:18.16e}:  Negative temperatures are "
                  "prohibited!".format(
                        self.T))

        if self.R < 0.0:
            raise ValueError(
                  "R = {0:18.16e}:  Negative ideal gas constant is "
                  "prohibited!".format(
                        self.R))

        if not np.isreal(self.b):
            raise ValueError('Modified Arrhenius parameter b must be real!')

        else:
            try:
                self.k = self.A * np.power(self.T, self.b) * np.exp(
                      -self.E / (self.R * self.T))
            except Warning:
                raise OverflowError("The result is too large/small.")

            return self.k

class BackwardCoefficient():
    """ Class of BackwardCoefficient
    """

    def __init__ (self, species, T, ki, is_reversible, vi_p, vi_dp, db_name='NASA_coef.sqlite'):
        self.p0 = 10e5
        self.R = 8.314
        self.species = species
        self.T = T
        self.ki = ki
        self.b_ki = np.zeros((len(is_reversible,)))
        self.is_reversible = is_reversible
        self.vi_p = vi_p
        self.vi_dp = vi_dp
        self.vi = np.matrix(self.vi_dp).T - np.matrix(self.vi_p).T  # calculate overall stoicheometric coefficients
        self.gamma = np.sum(self.vi, axis=0)
        self.dao = ThermoDAO(db_name)


    def get_backward_coefs (self):
        species_high = self.dao.get_species(self.T, 'high')
        species_low = self.dao.get_species(self.T, 'low')

        H_over_RT_species = []
        S_over_R_species = []
        for s in self.species:
            if self.T >= 1000: # high temperature range
                if s not in species_high:
                    raise ChemKinError('Thermo.get_backward_coefs()',
                        'The specie {}\'s high temperature bound is not higher than the current T={}.'.format(s, self.T))
                NASA_coefs = self.dao.get_coeffs(s, 'high')
            else: # low temperature range
                if s not in species_low:
                    raise ChemKinError('Thermo.get_backward_coefs()',
                        'The specie {}\'s low temperature bound is not lower than the current T={}.'.format(s, self.T))
                NASA_coefs = self.dao.get_coeffs(s, 'low')

            # Compute enthalpy/(RT) of a specie
            H_T_arr = np.array([1, 1/2*self.T, 1/3*(self.T**2), 1/4*(self.T**3), 1/5*(self.T**4), 1/self.T])
            H_coef_arr = np.array([NASA_coefs[0], NASA_coefs[1], NASA_coefs[2], NASA_coefs[3], NASA_coefs[4], NASA_coefs[5]])
            H_over_RT_species.append(np.dot(H_T_arr, H_coef_arr))

            # Compute entropy/R of a specie
            S_T_arr = np.array([np.log(self.T), self.T, 1/2*(self.T**2), 1/3*(self.T**3), 1/4*(self.T**4), 1])
            S_coef_arr = np.array([NASA_coefs[0], NASA_coefs[1], NASA_coefs[2], NASA_coefs[3], NASA_coefs[4], NASA_coefs[6]])
            S_over_R_species.append(np.dot(S_T_arr, S_coef_arr))

        delta_H_over_RT = np.squeeze(np.asarray(np.dot(H_over_RT_species, self.vi))) # delta enthalpy of a system of reactions
        delta_S_over_R = np.squeeze(np.asarray(np.dot(S_over_R_species, self.vi))) # delta entropy of a system of reactions

        factor = np.asarray(np.power(self.p0 / (self.R * self.T), self.gamma))[0]

        for index, rxn_is_rev in enumerate(self.is_reversible):
            if rxn_is_rev:
                ke = factor[index] * np.exp(delta_S_over_R[index] - delta_H_over_RT[index])

                self.b_ki[index] = self.ki[index]/ke

        return self.b_ki
        
if __name__ == "__main__":
    import doctest
    doctest.testmod(verbose=True)


###############################################################################
# Tests for RxnData and XmlParser classes.

# Class methods are used on different XML files located in xml-files directory.
# Each version of the reactions XML file tests different features / cases:
#   rxns_ideal: Contains data in correct manner (the "ideal" example file).
#   rxns_neg_A: Same as rxns_ideal except for negative A rateCoeff in first
#       reaction.
###############################################################################

from pytest import approx

from chemkin import pckg_xml_path
from chemkin.chemkin_errors import ChemKinError
from chemkin.preprocessing.parse_xml import RxnType, XmlParser


def test_parse_basic_functionality ():
    """Ensures number of reactions returned is correct and that attributes
    of the <reaction> element in the XML file are parsed correctly.
    """
    xml = XmlParser(pckg_xml_path('rxns_ideal.xml'))
    species, rxns = xml.load()

    # Correct number of reactions returned.
    err_msg = 'Expected 2 reactions but received {}.'.format(len(rxns))
    assert len(rxns) == 2, err_msg

    # Attributes of <reaction> element parsed properly.
    err_msg = 'reversible attribute not parsed properly.'
    assert rxns[0].reversible == False, err_msg
    assert rxns[1].reversible == False, err_msg

    err_msg = 'rxn_id attribute not parsed properly.'
    assert rxns[0].rxn_id == 'reaction01', err_msg
    assert rxns[1].rxn_id == 'reaction02', err_msg

    err_msg = 'type attribute not parsed properly.'
    assert rxns[0].type == RxnType.Elementary, err_msg
    assert rxns[1].type == RxnType.Elementary, err_msg


def test_parse_reactants_products ():
    """Ensures reactants and products parsed correctly."""
    xml = XmlParser(pckg_xml_path('rxns_ideal.xml'))
    species, rxns = xml.load()

    err_msg = 'reactants not parsed correctly.'
    assert rxns[0].reactants == {'H':1, 'O2':1}, err_msg
    assert rxns[1].reactants == {'H2':1, 'O':1}, err_msg

    err_msg = 'products not parsed correctly.'
    assert rxns[0].products == {'OH':1, 'O':1}, err_msg
    assert rxns[1].products == {'OH':1, 'H':1}, err_msg


def test_parse_rxn_coeff ():
    """Ensures reaction coefficients are what we expect them to be."""
    xml = XmlParser(pckg_xml_path('rxns.xml'))
    species, rxns = xml.load()

    rxn1_coeff = rxns[0].rate_coeff
    rxn2_coeff = rxns[1].rate_coeff
    rxn3_coeff = rxns[2].rate_coeff

    # reaction01
    assert rxn1_coeff[0] == approx(3.52e+10)
    assert rxn1_coeff[1] == approx(7.14e+04)

    # reaction02
    assert rxn2_coeff[0] == approx(5.06e-2)
    assert rxn2_coeff[1] == approx(2.7)
    assert rxn2_coeff[2] == approx(2.63e+04)

    # reaction03
    assert rxn3_coeff == approx(1.0e+03)


def test_rxndata_equation ():
    """Ensures equation representation of RxnData is correct."""
    xml = XmlParser(pckg_xml_path('rxns_ideal.xml'))
    species, rxns = xml.load()

    err_msg = 'equation() method result different than expected.'
    expected_0 = 'H + O2 [=] O + OH'
    print(rxns[0].equation())
    assert rxns[0].equation() == expected_0, err_msg

    err_msg = 'equation() method result different than expected.'
    expected_1 = 'H2 + O [=] H + OH'
    assert rxns[1].equation() == expected_1, err_msg


def test_badparse_negative_A_arr ():
    """Ensures ChemKinError raised when A coefficient for one of the
    reactions is negative.
    """
    xml = XmlParser(pckg_xml_path('rxns_neg_A_2'))
    try:
        rxns = xml.load()
    except ChemKinError as err:
        assert type(err) == ChemKinError
        assert str(err).find(
              'A coeff < 0 in reaction with id = reaction02') != -1


def test_badparse_negative_A_modarr ():
    """Ensures ChemKinError raised when A coefficient for one of the
    reactions is negative.
    """
    xml = XmlParser(pckg_xml_path('rxns_neg_A'))
    try:
        rxns = xml.load()
    except ChemKinError as err:
        assert type(err) == ChemKinError
        assert str(err).find(
              'A coeff < 0 in reaction with id = reaction01') != -1


"""
Contains class ODE_int_solver to compute the evolution of reaction species'
concentrations over specified time range.
"""
import numpy as np
from scipy.integrate import odeint


class ODE_int_solver():
    """Integrates the time evolution of a concentration of a specie i over
    specified time range.

    Attributes:
        temp (float): Temperature for reaction (assumed to be held constant).
        rxn (object): an instance of the ElementaryRxn() object
        species_equil_thresh (float, default 1e-5): Species concentration
            evolution  is defined to reach equilibrium once np.abs(bw - fw) <
            species_equil_thresh, where (bw - fw) is the difference in
            backward and forward progress rates for that species.
        overall_equil_thresh (float, default 1e-2): Overall reaction is
            defined to reach equilibrium once np.linalg.norm(prograte_diff) <
            overall_equil_thresh, where prograte_diff is the difference in
            the backward and forward progress rate vectors of the reaction.
        critical_t (list of floats of len(ki)): Stores time(s) at which
            reaction's component species' concentrations reach equilibrium.
        overall_critical_t (float): Stores time at which overall reaction
            reaches equilibrium
        max_t (float): maximum time allowed for the solver
        
    """

    def __init__ (self, temp, rxn, equil_thresh=1e-5,
                  overall_equil_thresh=1e-2, max_t=100):
        self.temp = temp
        self.rxn = rxn
        self.species_equil_thresh = equil_thresh
        self.overall_equil_thresh = overall_equil_thresh
        # Init critical time attrs with dummy values.
        self.critical_t = -100*np.ones((len(self.rxn.ki),))
        self.overall_critical_t = -100
        self.max_t = max_t

    def solve (self, time_int):
        """Solves evolution of specie concentration over specified time range.

        Args:
            time_int (list of floats): Time steps over which to solve evolution
                of species concentrations.

        Returns:
            sol (numpy array, shape (len(time_int), len(self.xi)): Results of
                calling scipy.integrate.odeint().
            self.critical_t
            self.overall_critical_t
        """
        i = 0  # iteration count

        def rxn_rate (x, t):
            nonlocal  i
            self.rxn.xi = x

            # reaction rate = 0 when some specie's concentration gets to zero
            n_species = len(x)
            for value in x:
                if value <= 0:
                    return np.zeros((n_species,))

            if i != 0:
                for j, (bw, fw) in enumerate(zip(self.rxn.b_wi, self.rxn.f_wi)):
                    if np.abs(bw - fw) < self.species_equil_thresh:
                        if self.critical_t[j] == -100:
                            self.critical_t[j] = t

                if np.linalg.norm(self.rxn.b_wi - self.rxn.f_wi) < self.overall_equil_thresh:
                    if self.overall_critical_t == -100:
                        self.overall_critical_t = t
            i += 1
            return self.rxn.reaction_rate()

        sol = odeint(func=rxn_rate, y0=self.rxn.xi, t=time_int, mxstep=5000000)
        return sol, self.critical_t, self.overall_critical_t


import os
import os.path
import numpy as np
import matplotlib
matplotlib.use('agg') # must be called before importing pyplot
import matplotlib.pyplot as plt
from chemkin.reaction.elementary_rxn import ElementaryRxn

def print_reaction_rate(parsed_data_list, xi):
	''' Function to print the reaction rates
	'''
	test_flag = 0 # reation rates can be printed
	
	for parsed_data in parsed_data_list:

		species = parsed_data['species']
		ki = parsed_data['ki']
		sys_vi_p = parsed_data['sys_vi_p']
		sys_vi_dp = parsed_data['sys_vi_dp']
		T = parsed_data['T']

		b_ki = parsed_data['b_ki']
		if str(b_ki) == 'Not Defined':
			test_flag = 1 # reaction rates cannot be printed because T is not in some specie's temperature range

			print('------At Temperature', T, 'K------')
			print('Backward reaction coefficients not defined: T={} is not in some specie\'s temperature range.'.format(T))
			print('--------------------------------')
			continue

		rxn_rates = ElementaryRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp).reaction_rate()

		print('------At Temperature', T, 'K------')
		for s, rate in zip(species, rxn_rates):
			print('    ', s, ':', rate)
		print('--------------------------------')

	return test_flag


def print_species_concentration(parsed_data_list, xi, n_steps=101, end_t=1e-12):
	''' Function to print the species concentration at a given time: end_t
	'''
	test_flag = 0 # species_concentrations can be printed
	for parsed_data in parsed_data_list:

		species = parsed_data['species']
		ki = parsed_data['ki']
		sys_vi_p = parsed_data['sys_vi_p']
		sys_vi_dp = parsed_data['sys_vi_dp']
		T = parsed_data['T']
		
		b_ki = parsed_data['b_ki']
		if str(b_ki) == 'Not Defined':
			test_flag = 1 # species_concentrations cannot be plotted because T is not in some specie's temperature range

			print('------At Temperature', T, 'K------')
			print('Backward reaction coefficients not defined: T={} is not in some specie\'s temperature range.'.format(T))
			print('--------------------------------\n')
			continue

		species_concentration = ElementaryRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp).species_concentration(T, end_t, n_steps)
		
		# print(np.min(sol), np.max(sol))

		print('------At Temperature', T, 'K------')
		print('Specie Concentration at time = {}'.format(end_t))
		for i, s in enumerate(species):
			print('  {}: start = {}, end = {}'.format(s, xi[i], species_concentration[i]))
		print('--------------------------------\n')

	return test_flag


def plot_species_concentration(parsed_data_list, xi, n_steps=101, end_t=1e-12):
	''' Function to plot the evolution of species concentration from start to an end time: end_t
	'''
	test_flag = 0 # species_concentrations can be plotted
	for parsed_data in parsed_data_list:

		species = parsed_data['species']
		ki = parsed_data['ki']
		sys_vi_p = parsed_data['sys_vi_p']
		sys_vi_dp = parsed_data['sys_vi_dp']
		T = parsed_data['T']
		
		b_ki = parsed_data['b_ki']
		if str(b_ki) == 'Not Defined':
			test_flag = 1 # species_concentrations cannot be plotted because T is not in some specie's temperature range

			print('------At Temperature', T, 'K------')
			print('Backward reaction coefficients not defined: T={} is not in some specie\'s temperature range.'.format(T))
			print('--------------------------------\n')
			continue

		time_steps = np.linspace(0, end_t, n_steps)
		species_concentration_evolution = ElementaryRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp).species_concentration_evolution(T, end_t, n_steps)
		

		f1, ax1 = plt.subplots(1, 1)
		for i, s in enumerate(species):
			# Plot the evolution of all species' concentration
			ax1.plot(time_steps, species_concentration_evolution[:, i], label='{}'.format(s))
		
		ax1.legend()
		ax1.set_ylim(0, np.max(species_concentration_evolution)+1)
		ax1.set_xlabel('Time (sec)')
		ax1.set_ylabel('Species Concentration')
		ax1.set_title('Evolution of Species Concentration over Time')

		BASE_DIR = os.path.dirname(os.path.abspath(__file__))
		image_dir = os.path.join(BASE_DIR, 'examples')
		if not os.path.exists(image_dir):
			os.makedirs(image_dir)
		img_path = os.path.join(image_dir, 'evolution_{}K.png'.format(T))
		f1.savefig(img_path)

	return test_flag


def print_time_to_equilibrium(parsed_data_list, xi, n_steps=101):
	''' Function to print the time to equilibrium of all reactions
	'''
	test_flag = 0 # time_to_equilibrium can be printed
	for parsed_data in parsed_data_list:

		ki = parsed_data['ki']
		sys_vi_p = parsed_data['sys_vi_p']
		sys_vi_dp = parsed_data['sys_vi_dp']
		T = parsed_data['T']
		
		b_ki = parsed_data['b_ki']
		if str(b_ki) == 'Not Defined':
			test_flag = 1 # time_to_equilibrium cannot be printed because T is not in some specie's temperature range

			print('------At Temperature', T, 'K------')
			print('Backward reaction coefficients not defined: T={} is not in some specie\'s temperature range.'.format(T))
			print('--------------------------------\n')
			continue

		end_t, critical_t, overall_critical_t = ElementaryRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp).time_to_equilibrium(T, n_steps)
		time_steps = np.linspace(0, end_t, n_steps)

		print('------At Temperature', T, 'K------')
		print('Time to Equilibrium (end_t = {})'.format(end_t))
		for i, t in enumerate(critical_t):
			if t == -100:
				t = 'Not yet reached equilibrium.'
			print('  Reaction #{}: {}'.format(i, t))

		print()
		if overall_critical_t == -100:
			overall_critical_t = 'The system has not yet reached equilibrium.'
		print('Overall Time to Equilibrium: {}'.format(overall_critical_t))
		print('--------------------------------\n')

	return test_flag



def plot_time_to_equilibrium(parsed_data_list, xi, n_steps=101):
	''' Function to plot the time to equilibrium of all reactions
	'''
	test_flag = 0 # time_to_equilibrium can be plotted
	for parsed_data in parsed_data_list:

		ki = parsed_data['ki']
		sys_vi_p = parsed_data['sys_vi_p']
		sys_vi_dp = parsed_data['sys_vi_dp']
		T = parsed_data['T']
		
		b_ki = parsed_data['b_ki']
		if str(b_ki) == 'Not Defined':
			test_flag = 1 # time_to_equilibrium cannot be plotted because T is not in some specie's temperature range

			print('------At Temperature', T, 'K------')
			print('Backward reaction coefficients not defined: T={} is not in some specie\'s temperature range.'.format(T))
			print('--------------------------------\n')
			continue

		end_t, critical_t, overall_critical_t = ElementaryRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp).time_to_equilibrium(T, n_steps)
		time_steps = np.linspace(0, end_t, n_steps)
		
		# Plot Log-scale Time to Equilibrium
		f2, ax2 = plt.subplots(1, 1)
		xpos = np.arange(0, len(parsed_data['equations']))
		
		# Take Log-transform of each rxn's time-to-equilibrium
		yval = np.copy(critical_t)
		for j, v in enumerate(yval):
			if v == -100:
				yval[j] = 0
			else:
				yval[j] = np.log10(v+1)
		labels = parsed_data['equations']

		for j, (x, y) in enumerate(zip(xpos, yval)):
			ax2.bar(x, y, width=0.5, bottom=0.0, align='center', alpha=0.6, label=labels[j]+':'+str('{:0.3e}'.format(y)))
		
		ax2.legend(fontsize=9)
		ax2.set_xlabel('Reaction equation')
		ax2.set_ylabel('log(Time + 1)')
		ax2.set_xticks(np.arange(0, len(parsed_data['equations'])))
		ax2.set_title('Log-scale Time to Equilibrium')

		BASE_DIR = os.path.dirname(os.path.abspath(__file__))
		image_dir = os.path.join(BASE_DIR, 'examples')
		if not os.path.exists(image_dir):
			os.makedirs(image_dir)
		img_path = os.path.join(image_dir, 'Time_to_Equilibrium_{}K.png'.format(T))
		f2.savefig(img_path)

	return test_flag






###############################################################################
# Tests for chemkin.viz.summary module
###############################################################################

import matplotlib.pyplot as plt
from chemkin import pckg_xml_path
from chemkin.preprocessing.parse_xml import XmlParser
from chemkin.viz import summary

def test_print_normal_irreversible():
	Ti = [2500]
	xi = [2.0, 1.0, 0.5, 1.0, 1.0]  # specie concentrations for 'rxns_hw5.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_hw5'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.print_reaction_rate(parsed_data_list, xi)
	assert test_flag == 0

def test_print_normal_reversible():
	Ti = [2500]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.print_reaction_rate(parsed_data_list, xi)
	assert test_flag == 0

def test_print_abnormal_reversible(): 
	Ti = [10000]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.print_reaction_rate(parsed_data_list, xi)
	assert test_flag == 1

def test_print_species_concentration_normal():
	Ti = [2500]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.print_species_concentration(parsed_data_list, xi)
	assert test_flag == 0

def test_print_species_concentration_abnormal():
	Ti = [10]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.print_species_concentration(parsed_data_list, xi)
	assert test_flag == 1


def test_plot_species_concentration_normal():
	Ti = [2500]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.plot_species_concentration(parsed_data_list, xi)
	assert test_flag == 0

def test_plot_species_concentration_abnormal():
	Ti = [10]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.plot_species_concentration(parsed_data_list, xi)
	assert test_flag == 1

def test_print_time_to_equilibrium_normal():
	Ti = [2500]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.print_time_to_equilibrium(parsed_data_list, xi)
	assert test_flag == 0

def test_print_time_to_equilibrium_abnormal():
	Ti = [10]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.print_time_to_equilibrium(parsed_data_list, xi)
	assert test_flag == 1

def test_plot_time_to_equilibrium_normal():
	Ti = [2500]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.plot_time_to_equilibrium(parsed_data_list, xi)
	assert test_flag == 0

def test_plot_time_to_equilibrium_abnormal():
	Ti = [10]
	xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti)
	test_flag = summary.plot_time_to_equilibrium(parsed_data_list, xi)
	assert test_flag == 1



"""
Test suite for the reaction_coefficients.py module

"""

import warnings

import numpy as np

from chemkin.reaction.reaction_coefficients import ArrheniusCoefficient, \
    ConstantCoefficient, ModifiedArrheniusCoefficient, RxnCoefficientBase, BackwardCoefficient
from chemkin import pckg_xml_path
from chemkin.preprocessing.parse_xml import XmlParser
from chemkin.chemkin_errors import ChemKinError


# tests for RxnCoefficientBase() base class
def test_base_get_coef ():
    try:
        RxnCoefficientBase().get_coef()
    except NotImplementedError as err:
        assert (type(err) == NotImplementedError)


def test_base_repr_ ():
    assert repr(RxnCoefficientBase()) == 'RxnCoefficientBase()'


# tests for ConstantCoefficient() class
def test_const_get_coef_result ():
    assert ConstantCoefficient(100).get_coef() == 100


def test_const_get_coef_neg_k ():
    try:
        ConstantCoefficient(-10).get_coef()
    except ValueError as err:
        assert (type(err) == ValueError)


def test_const_repr_ ():
    assert repr(ConstantCoefficient(5.0)) == 'ConstantCoefficient(k = 5.0)'


# tests for ArrheniusCoefficient() class
def test_arr_get_coef_result ():
    assert ArrheniusCoefficient(A=np.power(10, 4), E=100, T=300,
                                R=8.314).get_coef() == 9607.0007471344579


def test_arr_get_coef_neg_T ():
    try:
        ArrheniusCoefficient(A=np.power(10, 4), E=100, T=-300,
                             R=8.314).get_coef()
    except ValueError as err:
        assert (type(err) == ValueError)


def test_arr_get_coef_neg_A ():
    try:
        ArrheniusCoefficient(A=-np.power(10, 4), E=100, T=300,
                             R=8.314).get_coef()
    except ValueError as err:
        assert (type(err) == ValueError)


def test_arr_get_coef_neg_R ():
    try:
        ArrheniusCoefficient(A=np.power(10, 4), E=100, T=300,
                             R=-8.314).get_coef()
    except ValueError as err:
        assert (type(err) == ValueError)


def test_arr_get_coef_overflow ():
    with warnings.catch_warnings():
        warnings.filterwarnings('error')
        try:
            ArrheniusCoefficient(A=np.power(1, 256), E=-10000000., T=300,
                                 R=8.314).get_coef()
        except OverflowError as err:
            assert (str(err) == "The result is too large/small.")


def test_arr_repr_ ():
    assert repr(ArrheniusCoefficient(10, 100,
                                     150)) == 'ArrheniusCoefficient(A=10, ' \
                                              'E=100, T=150, R=8.314)'


# tests for ModifiedArrheniusCoefficient() class
def test_modarr_get_coef_result ():
    assert ModifiedArrheniusCoefficient(A=np.power(10, 4), b=0.5, E=100, T=300,
                                        R=8.314).get_coef() == \
           166398.13402389048


def test_modarr_get_coef_neg_T ():
    try:
        ModifiedArrheniusCoefficient(A=np.power(10, 4), b=0.5, E=100, T=-300,
                                     R=8.314).get_coef()
    except ValueError as err:
        assert (type(err) == ValueError)


def test_modarr_get_coef_neg_A ():
    try:
        ModifiedArrheniusCoefficient(A=-np.power(10, 4), b=0.5, E=100, T=300,
                                     R=8.314).get_coef()
    except ValueError as err:
        assert (type(err) == ValueError)


def test_modarr_get_coef_neg_R ():
    try:
        ModifiedArrheniusCoefficient(A=np.power(10, 4), b=0.5, E=100, T=300,
                                     R=-8.314).get_coef()
    except ValueError as err:
        assert (type(err) == ValueError)


def test_modarr_get_coef_complex_b ():
    try:
        ModifiedArrheniusCoefficient(A=np.power(10, 4), b=(3 + 1j), E=100,
                                     T=300, R=8.314).get_coef()
    except ValueError as err:
        assert (type(err) == ValueError)


def test_modarr_get_coef_overflow ():
    with warnings.catch_warnings():
        warnings.filterwarnings('error')
        try:
            ModifiedArrheniusCoefficient(A=np.power(10, 4), b=256.0, E=100,
                                         T=300, R=8.314).get_coef()
        except OverflowError as err:
            assert (str(err) == "The result is too large/small.")


def test_modarr_repr_ ():
    assert repr(ModifiedArrheniusCoefficient(A=10, b=0.5, E=100,
                                             T=300)) == 'ModifiedArrheniusCoefficient(A=10, b=0.5, E=100, T=300, R=8.314)'


def test_get_backward_coefs_normal():
    Ti = [750, 1500]
    xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
    parsed_data_list = xml_parser.parsed_data_list(Ti)  # calling Thermo().get_backward_coefs()
    assert parsed_data_list[0]['T'] == 750
    assert parsed_data_list[1]['T'] == 1500


def test_get_backward_coefs_high_range_err():
    Ti = [5000]
    xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
    parsed_data_list = xml_parser.parsed_data_list(Ti) # calling Thermo().get_backward_coefs()
    assert parsed_data_list[0]['b_ki'] == 'Not Defined'



def test_get_backward_coefs_low_range_err():
    Ti = [100]
    xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
    parsed_data_list = xml_parser.parsed_data_list(Ti) # calling Thermo().get_backward_coefs()
    assert parsed_data_list[0]['b_ki'] == 'Not Defined'



import os.path
import sqlite3


class ThermoDAO():
    """ Database Access Object for thermodynanmics
    """

    def __init__ (self, db_name):
        BASE_DIR = os.path.dirname(os.path.abspath(__file__))
        self.db_path = os.path.join(BASE_DIR, db_name)

    def get_coeffs (self, species_name, temp_range):
        db = sqlite3.connect(self.db_path)
        cursor = db.cursor()
        query = '''SELECT COEFF_1, COEFF_2, COEFF_3, COEFF_4, COEFF_5, COEFF_6, COEFF_7
                    FROM {} 
                    WHERE SPECIES_NAME = "{}"'''.format(temp_range.upper(), species_name)
        coeffs = list(cursor.execute(query).fetchall()[0])
        db.commit()
        db.close()
        return coeffs

    def get_species(self, temp, temp_range):
        db = sqlite3.connect(self.db_path)
        cursor = db.cursor()
        if temp_range == 'low': # temp_range == 'low'
            query = '''SELECT SPECIES_NAME FROM {} WHERE TLOW < {}'''.format(temp_range.upper(), temp)
        else: # temp_range == 'high'
            query = '''SELECT SPECIES_NAME FROM {} WHERE THIGH > {}'''.format(temp_range.upper(), temp)
        species = []
        for s in cursor.execute(query).fetchall():
            species.append(s[0])
        db.commit()
        db.close()
        return species

if __name__ == "__main__":
    import doctest
    doctest.testmod(verbose=True)


from setuptools import setup

setup(name='chemkin',
      version='3.4',
      description='The best chemkin package',
      url='https://github.com/G12-cs207-FinalProject/cs207-FinalProject.git',
      author='Monsieur Package du Parser',
      author_email='nathaniel_stein@g.harvard.edu',
      license='Harvard',
      packages=['chemkin',
                'chemkin.preprocessing',
                'chemkin.preprocessing.tests',
                'chemkin.reaction',
                'chemkin.reaction.tests',
                'chemkin.thermodynamics',
                'chemkin.thermodynamics.tests',
                'chemkin.viz',
                'chemkin.viz.tests',
                'chemkin.solver',
                'chemkin.solver.tests'],
      package_data={'chemkin':['thermodynamics/*.sqlite',
                               'xml-files/*.xml']})




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec  7 16:16:40 2017

@author: filipmichalsky
"""
#export PYTHONPATH="${PYTHONPATH}:/my/other/path"

#USE this
import sys
import numpy as np
#print(sys.path)
from chemkin import pckg_xml_path
from chemkin.preprocessing.parse_xml import XmlParser
from chemkin.solver.ODEint_solver import ODE_int_solver
from chemkin.reaction.elementary_rxn import ElementaryRxn

##########
#had to hack to make it work in my dir
#Ultimate hack is adding parent directory path to PYTHONPATH (only works in shell so far)
#import sys, os.path
#sys.path.append(os.path.abspath('../'))

#import ODEint_solver
#sys.path.remove(os.path.abspath('../'))
###############

def test_ODE_solver_functionality():
    
    """
    Test basic ODE solver functionality.
    """
    
    Ti = [1500]
    xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
    xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
    parsed_data_list = xml_parser.parsed_data_list(Ti)
    #print(parsed_data_list)
    for parsed_data in parsed_data_list:
        
        species = parsed_data['species']
        #print(species)
        ki = parsed_data['ki']
        sys_vi_p = parsed_data['sys_vi_p']
        sys_vi_dp = parsed_data['sys_vi_dp']
        T= parsed_data['T']
        b_ki=parsed_data['b_ki']

        rxn = ElementaryRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp)
        my_solver = ODE_int_solver(T, rxn)
        
        error_msg = "Solver is not initiating properly"
        assert my_solver != None and isinstance(my_solver,ODE_int_solver),error_msg
 
def test_ODE_solver_solve():
    
    """
    Tests ODE solver solve() method.
    """
    
    Ti = [1500]
    xi = [2., 1., .5, 1., 1., 1., .5, 1.] # specie concentrations 'rxns_reversible.xml'
    xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
    parsed_data_list = xml_parser.parsed_data_list(Ti)

    for parsed_data in parsed_data_list:
        
        species = parsed_data['species']
        ki = parsed_data['ki']
        sys_vi_p = parsed_data['sys_vi_p']
        sys_vi_dp = parsed_data['sys_vi_dp']
        T= parsed_data['T']
        b_ki=parsed_data['b_ki']       

        rxn = ElementaryRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp)
        my_solver = ODE_int_solver(T, rxn)

        time_int = time_steps = np.linspace(0, 100, 101)
        
        s , t_c , t_o =my_solver.solve(time_int)
        
        error_msg1 = "The dimensions of input do not match output"
        
        assert s.shape == (len(s),len(xi)),error_msg1
        
        assert len(t_c)==len(ki) and len(t_c)==len(b_ki),error_msg1
        
        #error_msg2 = "Reaction has not reached an equilibrium"
        #assert all(i>0 for i in t_c) and t_o >0, error_msg2
